/*
 * test_aes.c
 *
 *  Created on: 08-Dec-2021
 *      Author: ssinharoy
 */

#include<stdio.h>
#include<stdint.h>

extern volatile uint32_t * axi_address_base;

#include "../communication.h"
#include "../instruction.h"

#include "../aes/aes.h"

int test_aes()
{

	uint32_t i;
	uint64_t INS[64];
	uint64_t fpga_memory_data[1024]; // This is a memory buffer in SW. This buffer is used to read contents from the BRAM-memory of FPGA.

	// ---------------------- Example SW code for AES
	// AES-128 KEY GENERATION
	printf("---> Round keys are generated in SDK\n");
	uint64_t key_initial[2];
	uint64_t key_values[22];
	uint8_t key_rounds[176];

	key_initial[0] = 0xabf7158809cf4f3c; // low-bits
	key_initial[1] = 0x2b7e151628aed2a6; // high-bits

	key_gen(key_initial,key_values,key_rounds); // generate round-keys

	for(i=0; i<11; i++) {
		printf("Round-Key#%d: %llx",i,key_values[2*i+0]);
		printf("%llx\n",key_values[2*i+1]);
	}
	printf("\n");

	// AES-128 ENCRYPTION
	printf("---> Encryption is performed in SDK\n");
	uint64_t enc_plaintext[2];
	uint64_t enc_ciphertext[2];

	enc_plaintext[0]  = 0x313198a2e0370734; // low-bits
	enc_plaintext[1]  = 0x3243f6a8885a308d; // high-bits

	aes128_enc(enc_plaintext,enc_ciphertext,key_rounds);

	printf("Plaintext :%llx"  , enc_plaintext[1]);
	printf("%llx\n", enc_plaintext[0]);
	printf("Ciphertext:%llx"  , enc_ciphertext[1]);
	printf("%llx\n\n", enc_ciphertext[0]);

	// AES-128 DECRYPTION
	printf("---> Decryption is performed in SDK\n");
	uint64_t dec_ciphertext[2];
	uint64_t dec_plaintext[2];

	dec_ciphertext[0] = 0xdc118597196a0b32; // low-bits
	dec_ciphertext[1] = 0x3925841d02dc09fb; // high-bits

	aes128_dec(dec_ciphertext,dec_plaintext,key_rounds);

	printf("Ciphertext:%llx"  , dec_ciphertext[1]);
	printf("%llx\n", dec_ciphertext[0]);
	printf("Plaintext :%llx"  , dec_plaintext[1]);
	printf("%llx\n\n", dec_plaintext[0]);

	// ---------------------- Example HW code for AES
	// Initialize to zero
	for(i=0; i<1024; i++){
		fpga_memory_data[i] = 0;
	}

	// load key (word#2 to word#23)
	for(i=2; i<24; i++) {
		fpga_memory_data[i] = key_values[i-2];
	}

	// ------------------------------------------- ENC in HW

	printf("---> Encryption will be performed in HW\n");

	// load plaintext (word#0 to word#1) for encryption
	fpga_memory_data[0]  = enc_plaintext[0];
	fpga_memory_data[1]  = enc_plaintext[1];

	printf("---> Loading data to FPGA\n");
	send64(fpga_memory_data, 0, 24, 0); // sending the first 24 words of fpga_memory_data
	delay(1000);

	printf("---> Sending instructions\n");
	init_INS_AES_Enc(INS, 0, 24);
	send64(INS, 0, 64, 1); // send INStructions to FPGA
	delay(1000);

	printf("---> Executing instructions\n");
	exeIns(); // Execute instructions

	printf("---> Read data from FPGA\n");
	receive64(fpga_memory_data, 0, 26);
	delay(1000);

	for(i=24; i<26; i++){
		printf("fpga_memory_data[%d] = %llx\n", i, fpga_memory_data[i]);
	}

	printf("---> Compare results\n");
	if(enc_ciphertext[0] == fpga_memory_data[24])
		printf("%llx == %llx --> OK\n", enc_ciphertext[0], fpga_memory_data[24]);
	else
		printf("%llx == %llx --> FAIL\n", enc_ciphertext[0], fpga_memory_data[24]);

	if(enc_ciphertext[1] == fpga_memory_data[25])
		printf("%llx == %llx --> OK\n\n", enc_ciphertext[1], fpga_memory_data[25]);
	else
		printf("%llx == %llx --> FAIL\n\n", enc_ciphertext[1], fpga_memory_data[25]);

	// ------------------------------------------- DEC in HW

	printf("---> Decryption will be performed in HW\n");

	// load data (word#0 to word#1) for encryption
	fpga_memory_data[0]  = dec_ciphertext[0];
	fpga_memory_data[1]  = dec_ciphertext[1];

	printf("---> Loading data to FPGA\n");
	send64(fpga_memory_data, 0, 24, 0); // sending the first 24 words of fpga_memory_data
	delay(1000);

	printf("---> Sending instructions\n");
	init_INS_AES_Dec(INS, 0, 24);
	send64(INS, 0, 64, 1); // send INStructions to FPGA
	delay(1000);

	printf("---> Executing instructions\n");
	exeIns(); // Execute instructions

	printf("---> Read data from FPGA\n");
	receive64(fpga_memory_data, 0, 26);
	delay(1000);

	for(i=24; i<26; i++){
		printf("fpga_memory_data[%d] = %llx\n", i, fpga_memory_data[i]);
	}

	printf("---> Compare results\n");
	if(dec_plaintext[0] == fpga_memory_data[24])
		printf("%llx == %llx --> OK\n", dec_plaintext[0], fpga_memory_data[24]);
	else
		printf("%llx == %llx --> FAIL\n", dec_plaintext[0], fpga_memory_data[24]);

	if(dec_plaintext[1] == fpga_memory_data[25])
		printf("%llx == %llx --> OK\n\n", dec_plaintext[1], fpga_memory_data[25]);
	else
		printf("%llx == %llx --> FAIL\n\n", dec_plaintext[1], fpga_memory_data[25]);



	return 0;
}


