#include<stdio.h>
#include<stdint.h>
#include<stdlib.h>

#include "pke_parameters.h"
#include "error_secret_polygen.h"
#include "poly_arithmetic.h"
#include "randombytes.h"

void indcpa_key_gen(uint16_t *pk, uint16_t *sk) 
{
	uint8_t seed_a[PKE_UNIFORM_SEEDBYTES], seed_e[PKE_NOISE_SEEDBYTES], seed_s[PKE_NOISE_SEEDBYTES];
	uint16_t a[PKE_N], e1[PKE_N];	
	uint16_t as[PKE_N];

	// Generate a random seed for the uniform polynomial a(x) 
	randombytes(seed_a, PKE_UNIFORM_SEEDBYTES);		// Currently this is a pseudo-random gen. Replace it by a TRNG-based function. 
	shake128(seed_a, PKE_UNIFORM_SEEDBYTES, seed_a, PKE_UNIFORM_SEEDBYTES); // Used for not revealing system pseudo-RNG state

	GenUniformPoly(a, seed_a);	// Generate a(x) with coefficients uniformly distributed./+

	// Generate a random seed for the error polynomial 
	randombytes(seed_e, PKE_NOISE_SEEDBYTES);		// Currently this is a pseudo-random gen. Replace it by a TRNG-based function. 
	shake128(seed_e, PKE_NOISE_SEEDBYTES, seed_e, PKE_NOISE_SEEDBYTES); // Used for not revealing system pseudo-RNG state

	GenErrorPoly(e1, seed_e);

	// Generate a random seed for the secret polynomial 
	randombytes(seed_s, PKE_NOISE_SEEDBYTES);		// Currently this is a pseudo-random gen. Replace it by a TRNG-based function. 
	shake128(seed_s, PKE_UNIFORM_SEEDBYTES, seed_s, PKE_UNIFORM_SEEDBYTES); // Used for not revealing system pseudo-RNG state

	GenSecretPoly(sk, seed_s);

	// as = a*s
	poly_mul(as, a, sk);

	// b = e1-a*s. It is stored in the first part of pk. 
	poly_sub(pk, e1, as);

	for(int i=0; i<PKE_UNIFORM_SEEDBYTES; i++)
		pk[PKE_N + i] = (uint16_t) seed_a[i];
}


void indcpa_encryption(uint8_t *message, uint16_t *pk, uint16_t *ct)
{
	uint8_t error_seeds[3*PKE_NOISE_SEEDBYTES];
	uint16_t a[PKE_N];
	uint8_t seed_a[PKE_UNIFORM_SEEDBYTES];

	uint16_t e1[PKE_N], e2[PKE_N], e3[PKE_N];
	uint16_t message_poly[PKE_N];

	// Extract seed_a from pk={b, seed_a}.  	
	for(int i=0; i<PKE_UNIFORM_SEEDBYTES; i++)
		seed_a[i] = (uint8_t) pk[PKE_N + i];

	GenUniformPoly(a, seed_a);	// Generate a(x) with coefficients uniformly distributed.


	BS2POLmsg(message, message_poly);	// Convert message byte-string into an encoded polynomial

	// Generate random seeds for three error polynomial 
	randombytes(error_seeds, 3*PKE_NOISE_SEEDBYTES);		// Currently this is a pseudo-random gen. Replace it by a TRNG-based function. 
	shake128(error_seeds, 3*PKE_NOISE_SEEDBYTES, error_seeds, 3*PKE_NOISE_SEEDBYTES); // Used for not revealing system pseudo-RNG state

	GenErrorPoly(e1, error_seeds);
	GenErrorPoly(e2, error_seeds+PKE_NOISE_SEEDBYTES);
	GenErrorPoly(e3, error_seeds+2*PKE_NOISE_SEEDBYTES);

	poly_add(e3, e3, message_poly); // e3 <-- e3 + message_poly


	// Now compute the ciphertext ct = {c1, c2}. In ct[] array, c2 is appended to c1. 	
	poly_mul(ct, a, e1); 	// c1 <-- a*e1
    
	poly_add(ct, e2, ct); // c1 <-- e2 + a*e1;
  

	poly_mul(ct+PKE_N, pk, e1); // c2 <-- b*e1

	poly_add(ct+PKE_N, e3, ct+PKE_N); // c2<-- e3 + b*e1

}

void indcpa_decryption(uint16_t *sk, uint16_t *ct, uint8_t *message)
{
	uint16_t message_poly[PKE_N];

	poly_mul(ct, ct, sk);	// c1 <-- c1*r2
	poly_add(message_poly, ct, ct+PKE_N); // c1 <-- c1*r2 + c2
	POLmsg2BS(message_poly, message);
}

