/*
 * test_rng.c
 *
 *  Created on: 08-Dec-2021
 *      Author: ssinharoy
 */


#include<stdio.h>
#include<stdint.h>

extern volatile uint32_t * axi_address_base;

#include "../communication.h"
#include "../instruction.h"

#define SAMPLE_SIZE 1000000

// TRNG-related files
#include "utils.h"
#include "most_common.h"
#include "collision_test.h"
#include "lz78y_test.h"
#include "multi_mmc_test.h"
#include "lag_test.h"
#include "multi_mcw_test.h"
#include "compression_test.h"
#include "markov_test.h"
#include "non_iid_tests.h"
#include "test_trng.h"

int test_trng()
{
	int COUNTER;
	uint32_t i;
	uint64_t INS[64];
	uint64_t fpga_memory_data[1024]; // This is a memory buffer in SW. This buffer is used to read contents from the BRAM-memory of FPGA.

	uint8_t *data = (uint8_t*) malloc(SAMPLE_SIZE*sizeof(uint8_t));

	// ---------------------- Example code for TRNG
	double min_entropy;
	uint32_t numer_of_ones = 0;


	init_INS_TRNG(INS, 512, 0);

	send64(INS, 0, 64, 1); // send INStructions to FPGA using send64() function.
	delay(1000);


	numer_of_ones = 0;

	for(COUNTER=0; COUNTER<31; COUNTER++){
		// Clear SW buffer before loading data in it from FPGA.
		for(i=0; i<1024; i++){
			fpga_memory_data[i] = 0;
		}

		// Step-i: Get 50*1000 bits of randomness in fpga_memory_data1
		printf("... Executing instructions\n");
		exeIns();
		printf("Executed all instructions!\n");
		delay(1000);
		receive64(fpga_memory_data, 0, 1024);
		delay(1000);


		// Step3: Convert received random words of fpga_memory_data into a bitstring.
		uint64_t temp;
		uint64_t i_mod64;
		for(i=0; i<64*512; i++){
			temp = fpga_memory_data[i/64];
			i_mod64 = i%64;
			temp = temp>>i_mod64;
			temp = temp & 1;
			if(i+64*512*COUNTER >=1000000) goto L1;
			data[i+64*512*COUNTER] = (uint8_t) temp;
			numer_of_ones = numer_of_ones + temp;
		}
	}

	L1:
	printf("Number of ones = %u\n", numer_of_ones);

	for(i=0; i<10; i++){
		printf("fpga_memory_data[%u] = %llx\n", i, fpga_memory_data[i]);
	}

	min_entropy = non_iid_tests(data);
	printf("Final min-entropy after all tests = %lf\n", min_entropy);

	//for(i=0; i<1024; i++)
	//	printf("fpga_memory_data[%d] = %llx\n", i, fpga_memory_data[i]);


	return 0;
}

